@@ -4,6 +4,7 @@ import collections
 import contextlib
 import pkg_resources
 import itertools
+import sys
 
 try:
     import cPickle as pickle
@@ -257,7 +258,7 @@ class Client(object):
     def __init__(self, host_list, encode_fn=pickle.dumps,
                     decode_fn=pickle.loads, compress_fn=None,
                     decompress_fn=None, max_threads=None,
-                    ch_replicas=100, default_encoding="utf-8"):
+                    ch_replicas=100, default_encoding="utf-8", max_value_size=1048576):
         """
         Create a new instance of the pymemc client.
         
@@ -277,6 +278,7 @@ class Client(object):
         self.threadpool = threadpool.ThreadPool(max_threads or len(host_list))
         self.hash = chash.ConsistentHash(replicas=ch_replicas)
         self.default_encoding = default_encoding
+        self.max_value_size = max_value_size
         # connect up sockets and add to chash
         for host_str in host_list:
             pool = connpool.SocketConnectionPool(self._parse_host(host_str))
@@ -361,8 +363,9 @@ class Client(object):
         >>> c.close()
         """
         self.quit()
-        for sock in self.hash.all_nodes():
-            sock.close()
+        for r in self.hash.all_nodes():
+            with connpool.pooled_connection(r) as sock:
+                sock.close()
 
     def _g_helper(self, key, socket_fn, failure_test, unpack=True, return_cas=False):
         """
@@ -398,6 +401,9 @@ class Client(object):
         else:
             flags = 0
         
+        if sys.getsizeof(val) > self.max_value_size:
+            return False
+
         key = self._encode_key(key)
         with self.sock4key(key) as sock:
             socksend(sock, socket_fn(key, val, expire, flags))
@@ -454,6 +460,14 @@ class Client(object):
         def per_host_fn(sisters_map, failure_list, hashkey=None):
             items = sisters_map.items()
             last_i = len(items)-1
+            
+            oversized_items = []
+            for item in items:
+                if sys.getsizeof(item) > self.max_value_size:
+                    oversized_items.append(item)
+            for item in oversized_items:
+                failure_list.append(item[0])
+                items.remove(item)
             with self.sock4key(hashkey or items[0][0]) as sock:
                 for i,(key,val) in enumerate(items):
                     flags, val = self._serialize(val)
@@ -773,11 +787,12 @@ class Client(object):
         >>> c.quit()
         True
         """
-        for sock in self.hash.all_nodes():
-            socksend(sock, _qnsv(M._quit))
-            (_, _, _, _, _, status, _, _, _, extra) = sockresponse(sock)
-            if status != R._no_error:
-                raise MemcachedError("%d: %s" % (status, extra))
+        for r in self.hash.all_nodes():
+            with connpool.pooled_connection(r) as sock:
+                socksend(sock, _qnsv(M._quit))
+                (_, _, _, _, _, status, _, _, _, extra) = sockresponse(sock)
+                if status != R._no_error:
+                    raise MemcachedError("%d: %s" % (status, extra))
         return True
 
     def flush_all(self, expire=0):
@@ -789,11 +804,12 @@ class Client(object):
         >>> c.flush_all()
         True
         """
-        for sock in self.hash.all_nodes():
-            socksend(sock, _f(M._flush, expire))
-            (_, _, _, _, _, status, _, _, _, extra) = sockresponse(sock)
-            if status != R._no_error:
-                raise MemcachedError("%d: %s" % (status, extra))
+        for r in self.hash.all_nodes():
+            with connpool.pooled_connection(r) as sock:
+                socksend(sock, _f(M._flush, expire))
+                (_, _, _, _, _, status, _, _, _, extra) = sockresponse(sock)
+                if status != R._no_error:
+                    raise MemcachedError("%d: %s" % (status, extra))
         return True
 
     def stats(self):
@@ -836,11 +852,12 @@ class Client(object):
         >>> c.noop()
         True
         """
-        for sock in self.hash.all_nodes():
-            socksend(sock, _qnsv(M._noop))
-            (_, _, _, _, _, status, _, _, _, extra) = sockresponse(sock)
-            if status != R._no_error:
-                raise MemcachedError("%d: %s" % (status, extra))
+        for r in self.hash.all_nodes():
+            with connpool.pooled_connection(r) as sock:
+                socksend(sock, _qnsv(M._noop))
+                (_, _, _, _, _, status, _, _, _, extra) = sockresponse(sock)
+                if status != R._no_error:
+                    raise MemcachedError("%d: %s" % (status, extra))
         return True
 
     def version(self):
@@ -863,8 +880,9 @@ class Client(object):
             host_key = "%s:%s" % sock.getpeername()
             rmap[host_key] = version_string
 
-        for sock in self.hash.all_nodes():
-            self.threadpool.add_task(per_host_version, sock, host_version_map)
+        for r in self.hash.all_nodes():
+            with connpool.pooled_connection(r) as sock:
+                self.threadpool.add_task(per_host_version, sock, host_version_map)
         self.threadpool.wait()
 
         return host_version_map
